<h3>Tips: Tasks</h3>
<hr>
<p>This panel contains all the tasks and their results on your hosts in your playbook.</p>
<p>Click on the action to view the task file in which that action is located.</p>
<p>Click on the task status in order to view the detailed results of that task.</p>
<p>
    All table columns are sortable. This allows you to, for example:
    <ul>
        <li>Sort tasks by duration to find which tasks took the longest time</li>
        <li>Sort (or reverse) the elapsed column to have the beginning of the playbook (or the end) first</li>
    </ul>
</p>
<p>
    The search box searches through all fields as well as task tags. This allows you to search, for example:
    <ul>
        <li><strong>do something</strong> would match tasks with '<i>do something</i>' in their name</li>
        <li><strong>localhost</strong> would match tasks for the host '<i>localhost</i>'</li>
        <li><strong>get_url</strong> would match tasks that use the '<i>get_url</i>' Ansible module</li>
        <li><strong>failed</strong> would match tasks for which the status is '<i>failed</i>'</li>
        <li><strong>production</strong> would match tasks that have the tag '<i>production</i>'</li>
    </ul>
</p>
