{% if record.type == 'json' or record.type == 'dict' -%}
    {{ record.value | pygments_formatter | safe }}
{% elif record.type == 'list' -%}
    <ul class="text-left">
    {% for item in record.value -%}
        <li>{{ item }}</li>
    {%- endfor %}
    </ul>
{% elif record.type == 'url' -%}
    <span class="pull-left"><a href="{{ record.value }}" target="_blank">{{ record.value }}</a></span>
{% else -%}
    <pre>{{ record.value }}</pre>
{% endif -%}