{% include "head.html" %}
<body>
<div class="container-fluid">
    <div class="row">
        <div class="col-md-12">
          <h2>File: <strong>{{ file_.path }}</strong></h2>
          <p>Last updated: {{ file_.playbook.time_end | datefmt }}</p>
          <p>Ansible version: <strong>{{ file_.playbook.ansible_version }}</strong></p>
          {{ file_.content.content | yamlhighlight | safe }}
        </div>
    </div>
</div>
</body>
</html>
