{% macro render_tags(result) -%}
    This task has the following tags:
    <ul>
        {% for tag in result.task.tags | from_json -%}
        <li>{{ tag }}</li>
        {% endfor %}
    </ul>
{% endmacro %}
{% if result.task.tags and result.task.tags | from_json | length > 0 -%}
    <div class="pull-left" data-toggle="tooltip" data-placement="bottom" data-html="true" title="{{ render_tags(result) }}">{{ result.task.name }}</div>
{% else -%}
    <div class="pull-left">{{ result.task.name }}</div>
{% endif -%}