<h3>Tips: Records</h3>
<hr>
<p>This panel contains persistent data recorded from your playbooks with the <strong>ara_record</strong> Ansible module.</p>
<p><strong>ara_record</strong> is an Ansible module bundled with ARA that allows you to record and attach any arbitrary data that is displayed here.</p>
<p>
    Some examples of things that can be registered with ara_record:
    <ul>
        <li>The git version of your playbooks</li>
        <li>The host from which your playbook ran</li>
        <li>An URL to retrieve logs for the playbook</li>
        <li>Important variables or debug information obtained throughout your playbook</li>
    </ul>
</p>
<p>For more information on how to use <strong>ara_record</strong> (and <strong>ara_read</strong>), refer to the documentation.</p>
