{% include "head.html" %}
<!--
This template is not actually served or linked from within the web application.
-->
<body>
<div class="container-fluid">
    <div class="row">
        <div class="col-md-12">
            <table class="table table-striped table-bordered table-hover table-condensed">
                <thead>
                    <tr>
                        <th>Host</th>
                    </tr>
                </thead>
                <tbody>
                    {% for host in hosts -%}
                    {% if host.facts -%}
                    <tr>
                        <td><a href="{{ url_for('host.show_host', id=host.id) }}">{{ host.id }}</a></td>
                    </tr>
                    {% endif -%}
                    {% endfor -%}
                </tbody>
            </table>
        </div>
    </div>
</div>
</body>
</html>
