{% include "head.html" %}
<body>
<div class="container-fluid">
    <div class="row">
        <div class="col-md-12">
          <h2>Host facts: <strong>{{ host.name }}</strong></h2>
          <p>Last updated: {{ host.facts.timestamp | datefmt }}</p>
          <p>Ansible version: <strong>{{ host.playbook.ansible_version }}</strong></p>
          <table class="table table-striped table-bordered table-hover table-condensed facts">
              <thead>
                  <tr>
                      <th>Fact</th>
                      <th>Value</th>
                  </tr>
              </thead>
              <tbody>
                  {% for fact, value in facts -%}
                  <tr>
                      <td id="{{ fact }}"><a href="#{{ fact }}">{{ fact }}</a></td>
                      <td>{{ value | pygments_formatter | safe }}</td>
                  </tr>
                  {% endfor -%}
              </tbody>
          </table>
        </div>
    </div>
</div>
</body>
</html>
