{% extends "layout.html" %}

{% block content -%}
<!-- Begin about -->
<div class="container-fluid">
    <div class="col-md-custom col-md-offset-custom">
        <div class="row text-center">
            <br>
            <img src="{{ url_for('static', filename='images/logo-main.svg') }}" width="287" height="226" alt="ARA: Ansible Run Analysis" />
            <h1><strong>ARA - Ansible Run Analysis</strong></h1>
            <h2><a href="https://github.com/openstack/ara" target="_blank">ARA</a> records <a href="https://www.ansible.com/" target="_blank">Ansible</a> playbook runs seamlessly to make them easier to visualize, understand and troubleshoot.</h2>
            <h3>
                ARA doesn't run your playbooks for you: it integrates with Ansible as a callback plugin wherever it is.<br>
                Whether you are running Ansible from your personal laptop or a server,
                all you need to do is to <a href="https://ara.readthedocs.io/en/latest/installation.html" target="_blank">install ARA</a>,
                <a href="https://ara.readthedocs.io/en/latest/configuration.html" target="_blank">configure Ansible to use ARA</a> and you're good to go.
            </h3>
            <h3>
                The ARA project puts a lot of emphasis on being simple to install and use without requiring you change your existing Ansible workflows.<br>
                You can read about the project's core values and principles in the <a href="https://ara.readthedocs.io/en/latest/manifesto.html" target="_blank">project manifesto</a>.
            </h3>
            <br>
            {% if not empty_database -%}
            <h2>ARA makes the following data available:</h2>
            <br>
            <div class="list-group stat-highlight col-md-4 col-md-offset-4">
                <button type="button" class="list-group-item list-group-item-action">
                    <span class="badge">{{ playbooks }}</span>
                    Playbook runs
                </button>
                <button type="button" class="list-group-item list-group-item-action">
                    <span class="badge">{{ tasks }}</span>
                    Tasks
                </button>
                <button type="button" class="list-group-item list-group-item-action">
                    <span class="badge">{{ task_results }}</span>
                    Task results
                </button>
                <button type="button" class="list-group-item list-group-item-action">
                    <span class="badge">{{ hosts }}</span>
                    Hosts
                </button>
                <button type="button" class="list-group-item list-group-item-action">
                    <span class="badge">{{ host_facts }}</span>
                    Hosts with gathered facts
                </button>
                <button type="button" class="list-group-item list-group-item-action">
                    <span class="badge">{{ files }}</span>
                    Playbook, role and task files
                </button>
                <button type="button" class="list-group-item list-group-item-action">
                    <span class="badge">{{ records }}</span>
                    Recorded key/values
                </button>
            </div>
        </div>
        <div class="row text-center">
            <h2>Get started by looking at your <a href="{{ url_for('reports.report_list') }}">playbook reports</a>.</h2>
            {% else -%}
            <h2><strong>The ARA database is empty or the web application is not configured properly to find your playbook data.</strong></h2>
            <h2>Get started by <a href="https://ara.readthedocs.io/en/latest/configuration.html" target="_blank">configuring Ansible to use ARA</a>.</h2>
            <h2>Your data will be available in the interface after an <code>ansible-playbook</code> is run after configuring Ansible to use ARA.</h2>
            <br>
            <br>
                <h3>Need help ? Look at the <a href="https://ara.readthedocs.io/en/latest/" target="_blank">documentation</a> or come chat with the <a href="https://github.com/openstack/ara#community-and-getting-help" target="_blank">community</a> !</h3>
            {% endif -%}
        </div>
    </div>
</div>
<br>
<br>
<!-- End about -->
{% endblock %}
