{% extends "layout.html" %}

{% block content -%}
<!-- Begin single report -->
<div class="col-md-custom col-md-offset-custom">
    <div class="row">
        <div class="list-group list-view-pf list-view-pf-view">
            {% include "report.html" -%}
        </div>
    </div>
</div>
<!-- End single report -->
{% endblock -%}
